<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 08/29/2004 *
 * ===================                          Last change: 08/29/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : lib-mysql3.php                                   *
 * -------------------------------------------------------------------- *
 * Short description : Database layer for MySQL 3/4/5 server            *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Datenbankschicht fuer MySQL 3/4/5 Server         *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2012 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// SQL queries
function SQL_QUERY ($sqlString, $F, $L, $enableCodes = TRUE) {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__][$sqlString])) {
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Called: ' . $sqlString);

		// Trim SQL string
		$sqlStringModified = trim($sqlString);

		// Empty query string or link is not up?
		if (empty($sqlStringModified)) {
			// Empty SQL string!
			reportBug(__FUNCTION__, __LINE__, sprintf("SQL string is empty, please fix this: file=%s, line=%s",
				basename($F),
				$L
			));
		} elseif (!SQL_IS_LINK_UP()) {
			// We should not quietly ignore this
			reportBug(__FUNCTION__, __LINE__, sprintf("Cannot query database: sqlString=%s,file=%s,line=%s",
				$sqlStringModified,
				basename($F),
				$L
			));
		}

		// Remove \t, \n and \r from queries they may confuse some MySQL versions
		$sqlStringModified = str_replace(array(chr(9), PHP_EOL, chr(13)), array(' ', ' ', ' '), $sqlStringModified);

		// Compile config entries out
		$sqlStringModified = SQL_PREPARE_SQL_STRING($sqlStringModified, $enableCodes);

		// Cache it and remember as last SQL query
		$GLOBALS[__FUNCTION__][$sqlString] = $sqlStringModified;
		$GLOBALS['last_sql']               = $sqlStringModified;
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Stored cache: ' . $sqlStringModified);
	}  else {
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Cache used: ' . $sqlString);

		// Use cache (to save a lot function calls
		$GLOBALS['last_sql'] = $GLOBALS[__FUNCTION__][$sqlString];

		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Cache is: ' . $sqlString);
	}

	// Starting time
	$querytimeBefore = microtime(TRUE);

	// Run SQL command
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'F=' . basename($F) . ',L=' . $L . ',sql=' . $GLOBALS['last_sql']);
	$result = mysql_query($GLOBALS['last_sql'], SQL_GET_LINK())
		or SQL_ERROR($F, $L, 'file='. basename($F) . ',line=' . $L . ':mysql_error()=' . mysql_error() . ',last_query=' . $GLOBALS['last_sql']);
	//* DEBUG: */ logDebugMessage($F, $L, 'sql=' . $GLOBALS['last_sql'] . ',affected=' . SQL_AFFECTEDROWS() . ',numRows='.(is_resource($result) ? SQL_NUMROWS($result) : gettype($result)));

	// Calculate query time
	$queryTime = microtime(TRUE) - $querytimeBefore;

	// Add this query to array including timing
	addSqlToDebug($result, $GLOBALS['last_sql'], $queryTime, $F, $L);

	// Save last successfull query
	setConfigEntry('db_last_query', $GLOBALS['last_sql']);

	// Count all query times
	incrementConfigEntry('sql_time', $queryTime);

	// Count this query
	incrementConfigEntry('sql_count');

	// Debug output
	if (SQL_DEBUG_ENABLED()) {
		// Is this the first call?
		if (!isset($GLOBALS['sql_first_entry'])) {
			// Write first entry
			appendLineToFile(getCachePath() . 'mysql.log', 'Module=' . getModule());
			$GLOBALS['sql_first_entry'] = TRUE;
		} // END - if

		// Append debug line
		appendLineToFile(getCachePath() . 'mysql.log', basename($F) . '|LINE=' . $L . '|NUM=' . (is_resource($result) ? SQL_NUMROWS($result) : 'false') . '|AFFECTED=' . SQL_AFFECTEDROWS() . '|QUERYTIME:' . ($queryTime * 1000) . 'ms): ' . str_replace(array(chr(13), PHP_EOL), array('', ' '), $GLOBALS['last_sql']));
	} // END - if

	// Increment stats entry
	incrementStatsEntry('db_hits');

	// Return the result
	return $result;
}

// SQL num rows
function SQL_NUMROWS ($resource) {
	// Valid link resource?
	if (!SQL_IS_LINK_UP()) return FALSE;

	// Link is not up, no rows by default
	$lines = FALSE;

	// Is the result a valid resource?
	if (isset($GLOBALS['sql_numrows'][intval($resource)])) {
		// Use cache
		$lines = $GLOBALS['sql_numrows'][intval($resource)];
	} elseif (is_resource($resource)) {
		// Get the count of rows from database
		$lines = mysql_num_rows($resource);

		// Remember it in cache
		$GLOBALS['sql_numrows'][intval($resource)] = $lines;
	} else {
		// No resource given, please fix this
		reportBug(__FUNCTION__, __LINE__, 'No resource given! result[]=' . gettype($resource) . ',last_sql=' .  $GLOBALS['last_sql']);
	}

	// Return lines
	return $lines;
}

// SQL affected rows
function SQL_AFFECTEDROWS() {
	// Valid link resource?
	if (!SQL_IS_LINK_UP()) return FALSE;

	// Get affected rows
	$lines = mysql_affected_rows(SQL_GET_LINK());

	// Return it
	return $lines;
}

// SQL fetch row
function SQL_FETCHROW ($resource) {
	// Is a result resource set?
	if ((!is_resource($resource)) || (!SQL_IS_LINK_UP())) return FALSE;

	// Fetch the data and return it
	return mysql_fetch_row($resource);
}

// SQL fetch array
function SQL_FETCHARRAY ($resource) {
	// Is a result resource set?
	if ((!is_resource($resource)) || (!SQL_IS_LINK_UP())) return FALSE;

	// Load row from database
	$row = mysql_fetch_assoc($resource);

	// Return only arrays here
	if (is_array($row)) {
		// Return row
		return $row;
	} else {
		// Return a false, else some loops would go endless...
		return FALSE;
	}
}

// SQL result
function SQL_RESULT ($resource, $row, $field = '0') {
	// Is $resource valid?
	if ((!is_resource($resource)) || (!SQL_IS_LINK_UP())) return FALSE;

	// Run the result command
	$result = mysql_result($resource, $row, $field);

	// ... and return the result
	return $result;
}

// SQL connect
function SQL_CONNECT ($host, $login, $password, $F, $L) {
	// Try to connect
	$linkResource = mysql_connect($host, $login, $password) or SQL_ERROR($F, $L,  mysql_error());

	// Set the link resource
	if (is_resource($linkResource)) {
		/*
		 * A non-resource (boolean) may happen on installation phase which
		 * shall not be set here. Only valid link resources shall be set so
		 * SQL_IS_LINK_UP() will only return 'true' if there is really a
		 * working database link.
		 */
		SQL_SET_LINK($linkResource);
	} // END - if

	// Return the resource
	return $linkResource;
}

// SQL select database
function SQL_SELECT_DB ($dbName, $F, $L) {
	// Is there still a valid link? If not, skip it.
	if (!SQL_IS_LINK_UP()) return FALSE;

	// Return the result
	return mysql_select_db($dbName, SQL_GET_LINK()) or SQL_ERROR($F, $L,  mysql_error());
}

// SQL close link
function SQL_CLOSE ($F, $L) {
	if (!SQL_IS_LINK_UP()) {
		// Skip double close
		return FALSE;
	} // END - if

	// Close database link and forget the link
	$close = mysql_close(SQL_GET_LINK()) or SQL_ERROR($F, $L, mysql_error());

	// Close link
	SQL_SET_LINK(NULL);

	// Return the result
	return $close;
}

// SQL free result
function SQL_FREERESULT ($resource) {
	if ((!is_resource($resource)) || (!SQL_IS_LINK_UP())) {
		// Abort here
		return FALSE;
	} // END - if

	// Free result
	$res = mysql_free_result($resource);

	// And return that result of freeing it...
	return $res;
}

// Get id from last INSERT command
function SQL_INSERTID () {
	if (!SQL_IS_LINK_UP()) return FALSE;
	return mysql_insert_id();
}

// Escape a string for the database
function SQL_ESCAPE ($str, $secureString = TRUE, $strip = TRUE) {
	// Is there cache?
	if (!isset($GLOBALS['sql_escapes']['' . $str . ''])) {
		// Debug message
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'str=' . $str . ' - BEFORE!');

		// Prepare the string here
		$str = SQL_PREPARE_SQL_STRING($str);

		// Debug message
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'str=' . $str . ' - AFTER!');

		// Secure string first? (which is the default behaviour!)
		if ($secureString === TRUE) {
			// Debug message
			//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'str=' . $str . ',strip=' . intval($strip) . ' - BEFORE!');

			// Then do it here
			$str = secureString($str, $strip);

			// Debug message
			//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'str=' . $str . ',strip=' . intval($strip) . ' - AFTER!');
		} // END - if

		// Init (invalid) value
		$ret = '!INVALID!';

		if (!SQL_IS_LINK_UP()) {
			// Fall-back to escapeQuotes() when there is no link
			$ret = escapeQuotes($str);
		} elseif (function_exists('mysql_real_escape_string')) {
			// Debug message
			//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'str=' . $str);

			// The new and improved version
			$ret = mysql_real_escape_string($str, SQL_GET_LINK());

			// Debug message
			//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'str=' . $str . ',ret=' . $ret);
		} elseif (function_exists('mysql_escape_string')) {
			// The obsolete function
			$ret = mysql_escape_string($str, SQL_GET_LINK());
		} else {
			// If nothing else works, fall back to escapeQuotes() again
			$ret = escapeQuotes($str);
		}

		// Log message
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'str=' . $str . ',ret=' . $ret);

		// Cache result
		$GLOBALS['sql_escapes']['' . $str . ''] = $ret;
	} // END - if

	// Log message
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'str=' . $str . ',sql_escapes=' . $GLOBALS['sql_escapes']['' . $str . '']);

	// Return it
	return $GLOBALS['sql_escapes']['' . $str . ''];
}

// Log SQL errors to debug.log in installation phase or call reportBug()
function SQL_ERROR ($file, $line, $message) {
	// Remember plain error in last_sql_error
	$GLOBALS['last_sql_error'] = mysql_error();

	// Is there installation phase?
	if (isInstallationPhase()) {
		/*
		 * In installation phase, we don't want SQL errors abort e.g. connection
		 * tests, so just log it away.
		 */
		logDebugMessage($file, $line, $message);
	} else {
		// Regular mode, then call reportBug()
		reportBug($file, $line, $message);
	}
}

// [EOF]
?>
