<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 06/02/2011 *
 * ===================                          Last change: 06/02/2011 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : _filter.php                                      *
 * -------------------------------------------------------------------- *
 * Short description : Filters for ext-                                 *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Filter fuer ext-                                 *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2012 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Filter for generic handling of theme change
function FILTER_HANDLE_THEME_CHANGE ($filterData) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Called!');

	// Check if new theme is selcted
	if ((isPostRequestElementSet('new_theme')) && (postRequestElement('new_theme') != getCurrentTheme())) {
		// Set new theme for guests
		$newTheme = postRequestElement('new_theme');

		// Change to new theme
		setMailerTheme($newTheme);

		// Remove current from array
		removeIncludeFromPool('theme', sprintf("theme/%s/theme.php", getCurrentTheme()));

		// Add new theme
		addIncludeToPool('theme', sprintf("theme/%s/theme.php", $newTheme));

		// Redirect to same URL
		redirectToRequestUri();
	} // END - if

	// Return data
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Done!');
	return $filterData;
}

// Filter for settings theme from user profile, must be executed only if FILTER_FETCH_USER_DATA() ran before
function FILTER_SET_USERS_THEME ($filterData) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Called!');

	// Is the user data valid?
	if (!isMember()) {
		// Do only run for logged in members
		reportBug(__FUNCTION__, __LINE__, 'Please only run this filter for logged in users.');
	} // END - if

	// Change to new theme
	setMailerTheme(getUserData('curr_theme'));

	// Return data
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Done!');
	return $filterData;
}

// Filter for adding SQL columns on user registration about current theme
function FILTER_THEME_USER_REGISTRATION_ADD_SQL_COLUMNS ($filterData) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Called!');

	// Okay, add design here
	addExtraRegistrationColumns(', `curr_theme`');
	addExtraRegistrationData(", '{%%pipe,getCurrentTheme%%}'");

	// Return filter data
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Done!');
	//* NOISY-DEBUG: */ print __FUNCTION__.':filterData=<pre>'.print_r($filterData,TRUE).'</pre>';
	return $filterData;
}

// [EOF]
?>
