<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 07/13/2004 *
 * ===================                          Last change: 08/02/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : overview-inc.php                                 *
 * -------------------------------------------------------------------- *
 * Short description : Output standard task management                  *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Standard-Aufgaben-Management ausgeben            *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2012 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if ((!defined('__SECURITY')) || (!isAdmin())) {
	die();
} // END - if

// @TODO This function does also check for uncompleted tasks
function outputStandardOverview (&$result_tasks) {
	// First check for solved and not assigned tasks and assign them to current admin
	SQL_QUERY_ESC("UPDATE `{?_MYSQL_PREFIX?}_task_system` SET `assigned_admin`=%s WHERE `assigned_admin` IS NULL AND status != 'NEW'",
		array(getCurrentAdminId()), __FILE__, __LINE__);

	// We currently don't want to install an extension so let's find out if we need...
	setExtensionMode('test');
	$jobsDone = TRUE;

	// Open the extension directory
	$extensionList = getArrayFromDirectory('inc/extensions/', 'ext-', FALSE, FALSE);
	foreach ($extensionList as $file) {
		// Only file name is required... :(
		$file = basename($file);

		// Is this file an extension?
		if ((substr($file, 0, 4) == 'ext-') && (substr($file, -4) == '.php')) {
			// Possible newly installed extension found so we extract extension's name
			$ext_name = strtolower(substr($file, 4, -4)); // Keep always extension names on lower case!!!

			// Init variables
			$result = FALSE;

			// Check if extension is installed or not
			$extInstalled = (isExtensionInstalled($ext_name) && (isExtensionActive($ext_name)));

			// Is the extension not yet installed?
			//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'ext_name=' . $ext_name . ',extInstalled=' . intval($extInstalled));
			if ($extInstalled === FALSE) {
				// We maybe want to install an extension so let's test-drive it...
				if (loadExtension($ext_name, 'test', '0.0.0', TRUE)) {
					// Create a task for newly installed extension (we ignore the task id here)
					createNewExtensionTask($ext_name);
				} // END - if
			} else {
				// Test-drive extension
				loadExtension($ext_name, 'test');

				// Get extension version
				$ext_ver = getExtensionVersion($ext_name);
				//* DEBUG: */ debugOutput($ext_name.'='.$ext_ver);

				// Update extension if extension is installed and outdated
				//* DEBUG: */ debugOutput('ext=' . $ext_name . ',ver=' . getThisExtensionVersion() . '/' . getExtensionVersion($ext_name));
				if (getThisExtensionVersion() > $ext_ver) {
					// Update the extension
					updateExtension($ext_name, $ext_ver);
				} // END - if

				if (isset($GLOBALS['cache_array']['always_active'][$ext_name])) {
					// Shall the current extension kept activated?
					if (($GLOBALS['cache_array']['always_active'][$ext_name] == 'Y') && (!isExtensionActive($ext_name))) {
						// Reactivate this extension!
						doActivateExtension($ext_name);
					} // END - if
				} // END - if
			}
		} // END - if
	} // END - foreach

	// At last - but not least - check for own and new unassigned tasks...
	$result_tasks = SQL_QUERY_ESC("SELECT
	`id`,
	`assigned_admin`,
	`userid`,
	`task_type`,
	`subject`,
	`text`,
	`task_created`
FROM
	`{?_MYSQL_PREFIX?}_task_system`
WHERE
	`assigned_admin`=%s OR ((`assigned_admin`=0 OR `assigned_admin` IS NULL) AND `status`='NEW')
ORDER BY
	`userid` DESC,
	`task_type` DESC,
	`subject` ASC,
	`task_created` DESC",
		array(getCurrentAdminId()), __FILE__, __LINE__);

	if (!SQL_HASZERONUMS($result_tasks)) {
		// New jobs found
		$jobsDone = FALSE;
	} // END - if

	// Free the result
	SQL_FREERESULT($result);

	// Return status
	return $jobsDone;
}

// Outputs selected tasks
function outputSeletectedTasks ($postData, $result_tasks) {
	if ((isset($postData['assign'])) && (count($postData['sel']) > 0)) {
		// Assign / do tasks
		$OUT = '';
		foreach ($postData['sel'] as $taskId => $sel) {
			$result_task = SQL_QUERY_ESC("SELECT
	`id`,
	`userid`,
	`task_type`,
	`subject`,
	`text`,
	`task_created`,
	`status`,
	`assigned_admin`
FROM
	`{?_MYSQL_PREFIX?}_task_system`
WHERE
	`id`=%s AND (`assigned_admin`=%s OR ((`assigned_admin`=0 OR `assigned_admin` IS NULL) AND `status`='NEW'))
LIMIT 1",
				array(
					bigintval($taskId),
					getCurrentAdminId()
				), __FILE__, __LINE__);

			// Task is found?
			if (SQL_NUMROWS($result_task) == 1) {
				// Task is valid so load it's data
				$taskData = SQL_FETCHARRAY($result_task);

				if ($taskData['assigned_admin'] == '0') {
					// Assgin current admin to unassgigned task
					SQL_QUERY_ESC("UPDATE `{?_MYSQL_PREFIX?}_task_system` SET `assigned_admin`=%s WHERE `id`=%s AND `assigned_admin` IS NULL LIMIT 1",
						array(
							getCurrentAdminId(),
							bigintval($taskData['id'])
						), __FILE__, __LINE__);
				} // END - if

				$add = '';
				// @TODO Rewrite this to a filter
				if ($taskData['task_type'] == 'MEMBER_SUPPORT') {
					$mode = substr($taskData['text'], 0, strpos($taskData['text'], ':'));
					$taskData['text'] = substr($taskData['text'], strpos($taskData['text'], ':') + 1);
					$add = '<li>{--ADMIN_TASK_SUPPORT_MODE--}: <strong>' . $mode . '</strong></li>';
				} // END - if

				// Is a userid assign?
				if ($taskData['userid'] > 0) {
					// Then load his data!
					if (fetchUserData($taskData['userid'])) {
						// Fetch row
						$content = getUserDataArray();

						// Generate HTML list entry
						$add = '<li>{--ADMIN_MEMBER_USERID--}: <strong>' . generateUserProfileLink($taskData['userid'], 'user_data') . ' (<a href="' . generateEmailLink($content['email'], 'user_data') . '">{%pipe,translateGender=' . $content['gender'] . '%} ' . $content['surname'] . ' ' . $content['family'] . '</a>)</strong></li>';
					} else {
						// Invalid userid, so log and zero it
						logDebugMessage(__FUNCTION__, __LINE__, 'fetchUserData() failed: userid=' . $taskData['userid'] . ' not found.');
						$taskData['userid'] = '0';
					}
				} // END - if

				// Decode entities of the text
				$taskData['text'] = decodeEntities($taskData['text']);

				// Compile and insert text from task into table template
				$taskData['text'] = loadTemplate('admin_extensions_text', TRUE, $taskData['text']);

				// Initialize variables (no title for SQL commands by default)
				$ext_name = ''; $ext_ver = '';
				$title = '{--ADMIN_TASK_NO_SQL_TITLE--}';

				// Shall I list SQL commands assigned to an extension installation or update task?
				if ((isVerboseSqlEnabled()) || (!isExtensionInstalled('sql_patches'))) {
					// Extract extension name from subject
					$ext_name = substr($taskData['subject'], 1, strpos($taskData['subject'], ':') - 1);

					// Update task or extension task?
					if (($taskData['task_type'] == 'EXTENSION') && (!isExtensionInstalled($ext_name))) {
						// Load SQL commands for registering in dry-run
						registerExtension($ext_name, $taskId, TRUE);

						// Is this non-productive?
						if (isExtensionProductive() === FALSE) {
							// Issue warning
							$taskData['text'] = displayMessage('{%message,ADMIN_EXTENSION_IS_NON_PRODUCTIVE=' . $ext_name . '%}', TRUE) . $taskData['text'];
						} // END - if

						// Set current extension name
						setCurrentExtensionName($ext_name);

						// Add notes to text
						$taskData['text'] .= getExtensionNotes();

						// Set title
						$title = '{--ADMIN_SQLS_EXECUTED_ON_REGISTER--}';
					} elseif ($taskData['task_type'] == 'EXTENSION_UPDATE') {
						// Prepare extension name and version
						$ext_name = substr($ext_name, 7);
						$ext_name = substr($ext_name, 0, strpos($ext_name, '-'));
						$test = '[UPDATE-' . $ext_name . '-';
						$ext_ver = substr($taskData['subject'], strlen($test));
						$ext_ver = substr($ext_ver, 0, strpos($ext_ver, ':'));

						// Load SQLs from file
						updateExtension($ext_name, $ext_ver, TRUE);

						// Set current extension name
						setCurrentExtensionName($ext_name);

						// Add notes to text
						$taskData['text'] .= getExtensionNotes();

						// Set title
						$title = '{--ADMIN_SQLS_EXECUTED_ON_UPDATE--}';
					} else {
						// Remove extension's name
						$ext_name = '';
					}

					// Add SQLs to a table
					if ((!empty($ext_name)) && (isVerboseSqlEnabled())) {
						// Add verbose SQL table
						$taskData['text'] .= addExtensionVerboseSqlTable($title);
					} // END - if
				} elseif ((!empty($ext_name)) && (!empty($ext_ver))) {
					// Run SQL commands in dry mode but only return the notes
					updateExtension($ext_name, $ext_ver, TRUE);

					// Set current extension name
					setCurrentExtensionName($ext_name);

					// Get notes
					$taskData['text'] .= getExtensionNotes();
				} else {
					// This should not normally happen!
					reportBug(__FILE__, __LINE__, 'ext_name(' . $ext_name . ') or ext_ver(' . $ext_ver . ') is empty! isVerboseSqlEnabled=' . intval(isVerboseSqlEnabled()));
				}

				// Prepare array for the template
				$content = array(
					'subject'      => $taskData['subject'],
					'add'          => $add,
					'text'         => $taskData['text'],
					'task_created' => generateDateTime($taskData['task_created'], '1'),
					'ext_name'     => $ext_name
				);

				// Load template
				$OUT .= loadTemplate('admin_overview_row', TRUE, $content);

				// Which task do we actually have here?
				// @TODO Rewrite this to something with include files and/or filter
				switch ($taskData['task_type']) {
					case 'EXTENSION': // Install new extensions
						$ext_name = substr($taskData['subject'], 1, strpos($taskData['subject'], ':') - 1);
						if (!isExtensionInstalled($ext_name)) {
							// New (not yet installed) extension found
							$OUT .= loadTemplate('admin_extension_reg_form', TRUE, array(
								'id'       => bigintval($taskId),
								'ext_name' => $ext_name
							));
						} else {
							// Extension is already installed
							$OUT .= '<div class="notice">{--ADMIN_EXTENSION_ALREADY_REGISTERED--}</div>';

							// Close task but not already closed, solved, deleted or update tasks
							if ((!in_array($taskData['status'], array('CLOSED', 'DELETED', 'SOLVED'))) && ($taskData['task_type'] != 'EXTENSION_UPDATE')) {
								// Solve the task
								runFilterChain('solve_task', $taskData['id']);
							} // END - if
						}
						break;

					case 'EXTENSION_UPDATE': // Extension update
						// Extension updates are installed automatically
						$OUT .= '<div class="notice medium">{--ADMIN_EXTENSION_UPDATED--}</div>';

						// Close task if not closed or deleted
						if (!in_array($taskData['status'], array('CLOSED', 'DELETED'))) {
							// Solve the task
							runFilterChain('solve_task', $taskData['id']);
						} // END - if
						break;

					case 'MEMBER_SUPPORT': // Assign on member's support request
						// @TODO This may also be rewritten to include files
						switch ($mode) {
							default: // @TODO Unknown support mode
							logDebugMessage(__FILE__, __LINE__, sprintf("Unknown support mode %s detected. This part is under construction.", $mode));
							$OUT .= '<div class="notice medium">{%message,ADMIN_UNKNOWN_SUPPORT_MODE=' . $mode . '%}</div>';
							break;
						} // END - switch
						break;

					case 'PAYOUT_REQUEST': // Payout requests
						if (isExtensionActive('payout')) {
							// Extension is installed so let him send a notification to the user
							$result_pay = SQL_QUERY_ESC("SELECT `id` FROM `{?_MYSQL_PREFIX?}_user_payouts` WHERE `userid`=%s AND `payout_timestamp`=%s LIMIT 1",
								array(bigintval($taskData['userid']), bigintval($taskData['task_created'])), __FILE__, __LINE__);
							list($pid) = SQL_FETCHROW($result_pay);
							SQL_FREERESULT($result_pay);

							if ((!empty($pid)) && ($pid > 0)) {
								// Payout id can be obtained
								$content = array(
									'pid' => $pid,
									'tid' => $taskData['id'],
								);

								// Load template
								$OUT .= loadTemplate('admin_payout_overview_form', TRUE, $content);
							} else {
								// Problem obtaining payout id
								$OUT .= '<div class="notice medium">{--ADMIN_PAYOUT_OBTAIN_ID_FAILED--}</div>';
							}
						} else {
							// Extension is not installed
							$OUT .= '<div class="notice medium">{--ADMIN_PAYOUT_NOT_INSTALLED--}</div>';
						}
						break;

					case 'WERNIS_REQUEST': // Wernis requests
						if (isExtensionActive('wernis')) {
							// Extension is installed so let him send a notification to the user
							$result_pay = SQL_QUERY_ESC("SELECT `id` FROM `{?_MYSQL_PREFIX?}_user_wernis` WHERE `userid`=%s AND wernis_timestamp=%s LIMIT 1",
								array(bigintval($taskData['userid']), bigintval($taskData['task_created'])), __FILE__, __LINE__);
							list($pid) = SQL_FETCHROW($result_pay);
							SQL_FREERESULT($result_pay);

							if ((!empty($pid)) && ($pid > 0)) {
								// Payout id can be obtained
								$content = array(
									'pid' => $pid,
									'tid' => $taskData['id'],
								);
								$OUT .= loadTemplate('admin_wernis_overview_form', TRUE, $content);
							} else {
								// Problem obtaining wernis id
								$OUT .= '<div class="notice medium">{--WERNIS_OBTAIN_ID_FAILED--}</div>';
							}
						} else {
							// Extension is not installed
							$OUT .= '<div class="notice medium">{--ADMIN_WERNIS_NOT_INSTALLED--}</div>';
						}
						break;

					case 'HOLIDAY_REQUEST': // Holiday requests
						$OUT .= loadTemplate('admin_task_holiday', TRUE, $taskData['userid']);
						break;

					case 'MEMBER_ORDER': // Member mail orders
						$OUT .= loadTemplate('admin_task_order', TRUE, $taskId);
						break;

					default: // Unknown task type
						logDebugMessage(__FILE__, __LINE__, sprintf("Unknown task type %s detected.", $taskData['task_type']));
						$OUT .= '<div class="notice medium">' . sprintf(getMessage('ADMIN_UNKNOWN_TASK_TYPE'), $taskData['task_type'], $taskId) . '</div>';
						break;
				}
				$OUT .= '
  </td>
</tr>';
			} // END - if

			// Free result
			SQL_FREERESULT($result_task);
		} // END - foreach

		// Load final template
		loadTemplate('admin_overview_list', FALSE, $OUT);
	} else {
		if ((isset($postData['sel'])) && ((count($postData['sel']) > 0) || ($postData['sel'][0] == 1))) {
			// Only unassign / delete tasks when there are selected tasks posted
			if (!empty($postData['unassign'])) {
				// Unassign from tasks
				foreach ($postData['sel'] as $taskId => $sel) {
					SQL_QUERY_ESC("UPDATE `{?_MYSQL_PREFIX?}_task_system` SET `assigned_admin`=NULL WHERE `id`=%s AND `assigned_admin`=%s LIMIT 1",
						array(bigintval($taskId), getCurrentAdminId()), __FILE__, __LINE__);
				} // END - foreach
			} elseif (!empty($postData['delete'])) {
				// Delete tasks
				foreach ($postData['sel'] as $taskId => $sel) {
					SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM `{?_MYSQL_PREFIX?}_task_system` WHERE `id`=%s AND `assigned_admin` IN (%s,0,NULL) LIMIT 1",
						array(bigintval($taskId), getCurrentAdminId()), __FILE__, __LINE__);
				} // END - foreach
			} else {
				// Unknown action
				reportBug(__FILE__, __LINE__, sprintf("Unknown task action performed. data=<pre>%s</pre>", print_r($postData, TRUE)));
			}

			// Update query
			$result_tasks = SQL_QUERY_ESC("SELECT `id`, `assigned_admin`, `userid`, `task_type`, `subject`, `text`, `task_created` FROM `{?_MYSQL_PREFIX?}_task_system` WHERE `assigned_admin`=%s OR ((`assigned_admin`=0 OR `assigned_admin` IS NULL) AND `status`='NEW') ORDER BY `task_created` DESC",
				array(getCurrentAdminId()), __FILE__, __LINE__);
		} // END - if

		// There are uncompleted jobs!
		$OUT = '';
		while ($content = SQL_FETCHARRAY($result_tasks)) {
			// Init infos
			$content['infos'] = '';

			// Generate infos
			// @TODO Try to move this in includes
			switch ($content['task_type']) {
				case 'EXTENSION':
				case 'EXTENSION_UPDATE':
					$content['infos'] = substr($content['subject'], 1, strpos($content['subject'], ':') - 1);
					break;
			} // END - switch

			if (isValidUserId($content['userid'])) {
				// Member found otherwise it's a system task
				$content['userid'] = generateUserProfileLink($content['userid']);
			} else {
				$content['userid'] = '{--ADMIN_IS_SYSTEM_TASK--}';
			}

			// Add/translate some content
			$content['task_created'] = generateDateTime($content['task_created'], 2);

			// Is there extension task?
			if (isExtensionTask($content)) {
				// Load extension row template
				$OUT .= loadTemplate('admin_overview_list_ext_rows', TRUE, $content);
			} else {
				// Load default row template
				$OUT .= loadTemplate('admin_overview_list_rows', TRUE, $content);
			}
		} // END - while

		// Free memory
		SQL_FREERESULT($result_tasks);

		// Load footer template
		loadTemplate('admin_overview_table', FALSE, $OUT);
	}
}

// [EOF]
?>
